@@ -1,5 +1,5 @@
--- Internal.hs: private utility functions
--- Copyright © 2012-2015  Clint Adams
+-- Internal.hs: private utility functions and such
+-- Copyright © 2012-2016  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -7,10 +7,7 @@
 
 module Codec.Encryption.OpenPGP.Internal (
    countBits
- , beBSToInteger
- , integerToBEBS
  , PktStreamContext(..)
- , hashDescr
  , issuer
  , emptyPSC
  , pubkeyToMPIs
@@ -19,16 +16,25 @@ module Codec.Encryption.OpenPGP.Internal (
  , sigPKA
  , sigHA
  , sigCT
+ , truncatingVerify
 ) where
 
-import Crypto.PubKey.HashDescr (HashDescr(..), hashDescrMD5, hashDescrSHA1, hashDescrSHA224, hashDescrSHA256, hashDescrSHA384, hashDescrSHA512, hashDescrRIPEMD160)
+import Crypto.Hash (hashWith)
+import qualified Crypto.Hash.IO as CHI
+import Crypto.Number.Basic (numBits)
+import Crypto.Number.ModArithmetic (expFast, inverse)
+import Crypto.Number.Serialize (os2ip)
 import qualified Crypto.PubKey.DSA as DSA
 import qualified Crypto.PubKey.RSA as RSA
 
-import Data.Bits (testBit, shiftL, shiftR, (.&.))
+import Data.Bits (testBit)
+import Data.ByteArray (ByteArrayAccess)
+import qualified Data.ByteArray as BA
+import qualified Data.ByteString as B
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString.Lazy as BL
-import Data.List (find, mapAccumR, unfoldr)
+import Data.List (find)
+import Data.Maybe (fromJust)
 import Data.Word (Word8, Word16)
 
 import Codec.Encryption.OpenPGP.Types
@@ -42,12 +48,6 @@ countBits bs
         go _ 0 = 7
         go n b = if testBit n b then 7 - fromIntegral b else go n (b-1)
 
-beBSToInteger :: ByteString -> Integer
-beBSToInteger = sum . snd . mapAccumR (\acc x -> (acc + 8, fromIntegral x `shiftL` acc)) 0 . BL.unpack
-
-integerToBEBS :: Integer -> ByteString
-integerToBEBS = BL.pack . reverse . unfoldr (\x -> if x == 0 then Nothing else Just ((fromIntegral x :: Word8) .&. 0xff, x `shiftR` 8))
-
 data PktStreamContext = PktStreamContext { lastLD :: Pkt
                       , lastUIDorUAt :: Pkt
                       , lastSig :: Pkt
@@ -65,16 +65,6 @@ issuer (SignaturePkt (SigV4 _ _ _ _ usubs _ _)) = fmap (\(SigSubPacket _ (Issuer
         isIssuer _ = False
 issuer _ = Nothing
 
-hashDescr :: HashAlgorithm -> Either String HashDescr
-hashDescr SHA1 = Right hashDescrSHA1
-hashDescr RIPEMD160 = Right hashDescrRIPEMD160
-hashDescr SHA256 = Right hashDescrSHA256
-hashDescr SHA384 = Right hashDescrSHA384
-hashDescr SHA512 = Right hashDescrSHA512
-hashDescr SHA224 = Right hashDescrSHA224
-hashDescr DeprecatedMD5 = Right hashDescrMD5
-hashDescr x = Left $ "Unknown hash problem: " ++ show x
-
 pubkeyToMPIs :: PKey -> [MPI]
 pubkeyToMPIs (RSAPubKey (RSA_PublicKey k)) = [MPI (RSA.public_n k), MPI (RSA.public_e k)]
 pubkeyToMPIs (DSAPubKey (DSA_PublicKey k)) = [
@@ -82,7 +72,7 @@ pubkeyToMPIs (DSAPubKey (DSA_PublicKey k)) = [
                              , pkParams DSA.params_q
                              , pkParams DSA.params_g
                              , MPI . DSA.public_y $ k
-			     ]
+                             ]
   where pkParams f = MPI . f . DSA.public_params $ k
 
 pubkeyToMPIs (ElGamalPubKey k) = fmap MPI k
@@ -115,4 +105,18 @@ sigCT (SigV4 _ _ _ hsubs _ _ _) = fmap (\(SigSubPacket _ (SigCreationTime i)) ->
         isSigCreationTime _ = False
 sigCT _ = Nothing
 
+truncatingVerify :: (ByteArrayAccess msg, CHI.HashAlgorithm hash) => hash -> DSA.PublicKey -> DSA.Signature -> msg -> Bool
+truncatingVerify hashAlg pk (DSA.Signature r s) m
+    -- Reject the signature if either 0 < r < q or 0 < s < q is not satisfied.
+    | r <= 0 || r >= q || s <= 0 || s >= q = False
+    | otherwise                            = v == r
+    where (DSA.Params p g q) = DSA.public_params pk
+          y       = DSA.public_y pk
+          hm      = os2ip . dsaTruncate . BA.convert $ hashWith hashAlg m
 
+          w       = fromJust $ inverse s q
+          u1      = (hm*w) `mod` q
+          u2      = (r*w) `mod` q
+          v       = ((expFast g u1 p) * (expFast y u2 p)) `mod` p `mod` q
+          dsaTruncate bs = let lbs = BL.fromStrict bs in if countBits lbs > fromIntegral dsaQLen then B.take (dsaQLen `div` 8) bs else bs -- FIXME: uneven bits
+          dsaQLen = numBits $ q
