@@ -10,12 +10,12 @@ module Codec.Encryption.OpenPGP.KeySelection (
  , parseFingerprint
 ) where
 
-import qualified Data.ByteString.Lazy as BL
-import Codec.Encryption.OpenPGP.Internal (integerToBEBS)
 import Codec.Encryption.OpenPGP.Types
 import Control.Applicative (optional, (<$>), (*>))
 import Control.Monad ((<=<))
+import Crypto.Number.Serialize (i2osp)
 import Data.Attoparsec.Text (asciiCI, count, hexadecimal, inClass, parseOnly, Parser, satisfy)
+import qualified Data.ByteString.Lazy as BL
 import Data.Text (Text, toUpper)
 import qualified Data.Text as T
 
@@ -32,4 +32,4 @@ hexen :: Int -> Parser Text
 hexen n = T.pack <$> count n (satisfy (inClass "A-F0-9"))
 
 hexes :: Parser BL.ByteString
-hexes = integerToBEBS <$> hexadecimal
+hexes = BL.fromStrict . i2osp <$> hexadecimal
