@@ -1,5 +1,5 @@
 -- Serialize.hs: OpenPGP (RFC4880) serialization (using cereal)
--- Copyright © 2012-2015  Clint Adams
+-- Copyright © 2012-2016  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -11,6 +11,8 @@ module Codec.Encryption.OpenPGP.Serialize (
 import Control.Applicative ((<$>), many, some)
 import Control.Lens ((^.), _1)
 import Control.Monad (guard, liftM, replicateM, replicateM_)
+import Crypto.Number.Basic (numBits)
+import Crypto.Number.Serialize (i2osp, os2ip)
 import qualified Crypto.PubKey.RSA as R
 import qualified Crypto.PubKey.DSA as D
 import Data.Bits ((.&.), (.|.), shiftL, shiftR, testBit)
@@ -33,7 +35,7 @@ import Data.Word (Word8, Word32)
 import Data.Maybe (fromMaybe)
 import Network.URI (nullURI, parseURI, uriToString)
 
-import Codec.Encryption.OpenPGP.Internal (countBits, beBSToInteger, integerToBEBS, pubkeyToMPIs, multiplicativeInverse)
+import Codec.Encryption.OpenPGP.Internal (countBits, pubkeyToMPIs, multiplicativeInverse)
 import Codec.Encryption.OpenPGP.Types
 
 instance Binary SigSubPacket where
@@ -752,13 +754,13 @@ putLengthThenPayload bs = do
 
 getMPI :: Get MPI
 getMPI = do mpilen <- getWord16be
-            bs <- getLazyByteString (fromIntegral (mpilen + 7) `div` 8)
-            return $ MPI (beBSToInteger bs)
+            bs <- getByteString (fromIntegral (mpilen + 7) `div` 8)
+            return $ MPI (os2ip bs)
 
 getPubkey :: PubKeyAlgorithm -> Get PKey
 getPubkey RSA = do MPI n <- get
                    MPI e <- get
-                   return $ RSAPubKey (RSA_PublicKey (R.PublicKey (fromIntegral . BL.length . integerToBEBS $ n) n e))
+                   return $ RSAPubKey (RSA_PublicKey (R.PublicKey (fromIntegral . B.length . i2osp $ n) n e))
 getPubkey DeprecatedRSAEncryptOnly = getPubkey RSA
 getPubkey DeprecatedRSASignOnly = getPubkey RSA
 getPubkey DSA = do MPI p <- get
@@ -801,9 +803,9 @@ putSKey (RSAPrivateKey (RSA_PrivateKey (R.PrivateKey _ d p q _ _ _))) = put (MPI
         u = multiplicativeInverse q p
 
 putMPI :: MPI -> Put
-putMPI (MPI i) = do let bs = integerToBEBS i
-                    putWord16be . countBits $ bs
-                    putLazyByteString bs
+putMPI (MPI i) = do let bs = i2osp i
+                    putWord16be . fromIntegral . numBits $ i
+                    putByteString bs
 
 getPKPayload :: Get PKPayload
 getPKPayload = do
