@@ -1,5 +1,5 @@
 -- Types.hs: OpenPGP (RFC4880) data types
--- Copyright © 2012-2015  Clint Adams
+-- Copyright © 2012-2016  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -27,6 +27,7 @@ import qualified Crypto.PubKey.ECC.ECDSA as ECDSA
 import Data.Aeson ((.=), object)
 import qualified Data.Aeson as A
 import Data.Byteable (Byteable)
+import Data.ByteArray (ByteArrayAccess)
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
@@ -570,13 +571,13 @@ instance FutureFlag KeyFlag where
 instance Hashable KeyFlag
 
 instance Pretty KeyFlag where
-    pretty GroupKey = text "group key"
-    pretty AuthKey = text "auth key"
-    pretty SplitKey = text "split key"
-    pretty EncryptStorageKey = text "encrypt-storage key"
-    pretty EncryptCommunicationsKey = text "encrypt-communications key"
-    pretty SignDataKey = text "sign-data key"
-    pretty CertifyKeysKey = text "certify-keys key"
+    pretty GroupKey = text "group"
+    pretty AuthKey = text "auth"
+    pretty SplitKey = text "split"
+    pretty EncryptStorageKey = text "encrypt-storage"
+    pretty EncryptCommunicationsKey = text "encrypt-communications"
+    pretty SignDataKey = text "sign-data"
+    pretty CertifyKeysKey = text "certify-keys"
     pretty (KFOther o) = text "unknown key flag type" <+> pretty o
 
 instance A.ToJSON KeyFlag
@@ -740,7 +741,7 @@ instance Pretty PKPayload where
 instance A.ToJSON PKPayload
 
 newtype IV = IV {unIV :: B.ByteString}
-    deriving (Byteable, Data, Eq, Generic, Hashable, Monoid, Show, Typeable)
+    deriving (Byteable, ByteArrayAccess, Data, Eq, Generic, Hashable, Monoid, Show, Typeable)
 
 instance Newtype IV B.ByteString where
     pack = IV
